/* From Alain : This program captures key events and prints the key modifier, 
 *              "group=0" key label (unicode char) and "key name" according to GDK
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

gboolean on_key_press(GtkWidget * widget, GdkEventKey * event, gpointer user_data);

int main(int argc, char *argv[])
{
	GtkWidget *window;

	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect(G_OBJECT(window), "key_press_event", G_CALLBACK(on_key_press), NULL);

	gtk_widget_show_all(window);

	gtk_main();

	return 0;
}

void key_press_cb(GdkEventKey * kev)
{

	if (kev->keyval <= 0xffff) {
		GdkModifierType state = (GdkModifierType) (kev->state);
		//int slen, mods = 0;
		//static int seq_len = 0;
		unsigned short int kv = kev->keyval;

		guint *keyvals;
		GdkKeymapKey *keys;
		gint n_entries;
		unsigned short int ukv;

		kv = gdk_keyval_to_lower(kev->keyval);
		ukv = gdk_keyval_to_unicode(kv);
		if (ukv < 'a' || ukv > 'z') {
			if (gdk_keymap_get_entries_for_keycode(gdk_keymap_get_default(), kev->hardware_keycode, &keys, &keyvals, &n_entries)) {
				for (int n = 0; n < n_entries; n++) {
					printf("     group=%d \tlevel=%d \t%c \t%s\n", keys[n].group, keys[n].level, gdk_keyval_to_unicode(keyvals[n]),
								 gdk_keyval_name(keyvals[n]));
				}
				kv = keyvals[0];
				//kv = keyvals[2 * kev->group];
				g_free(keys);
				g_free(keyvals);
			}
		}
		printf("Found: ");
		if (state & GDK_SHIFT_MASK) {
			printf("<shift>");
		}
		if (state & GDK_CONTROL_MASK) {
			printf("<ctrl>");
		}
		if (state & GDK_MOD1_MASK) {
			printf("<alt>");
		}
		printf(" %c \tkeyname = %s\n", gdk_keyval_to_unicode(kv), gdk_keyval_name(kv));

	}
}

gboolean on_key_press(GtkWidget * widget, GdkEventKey * event, gpointer user_data)
{
	key_press_cb(event);

	/* CTRL-q exits the application */
	if (event->keyval == GDK_KEY_q && event->state & GDK_CONTROL_MASK) {
		gtk_main_quit();
	}

	return FALSE;
}
